from django.http import HttpResponse, HttpResponseBadRequest
from django.core.files.storage import default_storage
from django.views.decorators.csrf import csrf_exempt
from django.utils import simplejson as json
from django.utils.encoding import force_unicode, smart_str

from directupload.backends import get_directupload_backend

from urlparse import parse_qsl
import os
import datetime

def upload_options_view(request):
    backend = get_directupload_backend()
    data = backend(request=request).get_options_json()
    return HttpResponse(data)

@csrf_exempt
def upload_file(request):
    #this is handled by a different session then the user's browser, hence csrf exempt
    if not request.POST:
        return HttpResponseBadRequest()
    from directupload.backends.djangoview import unsign
    data = dict(parse_qsl(unsign(request.POST['payload'])))
    assert data['request_time'] #TODO respect some expiration
    path = request.POST['targetpath']
    file_path = default_storage.save(path, request.FILES['file']) #TODO how to tell the storage engine not to rename?
    return HttpResponse(file_path)

def determine_name(request):
    if not request.POST:
        return HttpResponseBadRequest()
    
    upload_to = request.POST['upload_to']
    upload_to = os.path.normpath(force_unicode(datetime.datetime.now().strftime(smart_str(upload_to))))
    
    desired_path = os.path.join(upload_to, request.POST['filename'])
    if desired_path.startswith('/'):
        desired_path = desired_path[1:]
    path = default_storage.get_available_name(desired_path)
    folder = os.path.split(desired_path)[0]
    
    data = {'targetpath':path,
            'targetname':os.path.split(path)[-1],}
    backend = get_directupload_backend()
    backend(request=request, options={'folder':folder}).update_post_params(data)
    return HttpResponse(json.dumps(data))




from django import template
from django.db import models
from django.db.models.fields import FieldDoesNotExist

import copy

register = template.Library()

from classytags.core import Options
from classytags.helpers import InclusionTag
from classytags.arguments import Argument, KeywordArgument

from directupload.widgets import DirectUploadClearableFileInput

@register.inclusion_tag('directupload/templatetags/head.html')
def directupload_head():
    return {
        'media': DirectUploadClearableFileInput().media,
    }

class RenderDirectUploadField(InclusionTag):
    name = 'render_upload_field'
    template = 'directupload/templatetags/render_file_widget.html'
    options = Options(
        Argument('field', resolve=True),
        KeywordArgument('model', resolve=True, required=False),
        KeywordArgument('model_field', resolve=True, required=False),
    )
    
    def get_context(self, context, field, model, model_field):
        model = model.get('model', None)
        model_field = model_field.get('model_field', None)
        if not model_field and model:
            model_field = model._meta.get_field(field.name)
        if not model_field:
            pass
        widget = DirectUploadClearableFileInput(db_field=model_field)
        field.field = copy.copy(field.field) #looks weird but field is actually a bound field
        field.field.widget = widget
        return {'widget': widget, 'field':field}

register.tag(RenderDirectUploadField)

def is_file_field(field, model):
    #field|is_file_field:line.model_admin.model
    try:
        model_field = model._meta.get_field(field.name)
    except FieldDoesNotExist:
        return False
    return isinstance(model_field, models.FileField) or hasattr(model_field, 'upload_to')

register.filter('is_file_field', is_file_field)




from base import BaseUploadBackend

import datetime
from urllib import urlencode

from django.middleware.csrf import get_token
from django.core.urlresolvers import reverse

from utils import Signer
signer = Signer()

def sign(value):
    return signer.sign(value)

def unsign(value):
    return signer.unsign(value)

class DjangoViewBackend(BaseUploadBackend):
    """Uploadify for using the builtin django view"""
    
    def build_options(self):
        self.options['fileObjName'] = 'file'
    
    def get_target_url(self):
        return reverse('directupload-file')
    
    def build_post_data(self):
        data = {'upload_to': self.options['folder'],
                'request_time': datetime.datetime.now().isoformat(),
                '_request_id': 'foo',} #TODO salt}
        signed_data = sign(urlencode(data))
        self.post_data['payload'] = signed_data
        self.post_data['csrfmiddlewaretoken'] = get_token(self.request)




from django.conf import settings
from django.core.exceptions import ImproperlyConfigured
from urllib import quote_plus
from datetime import datetime
from datetime import timedelta
import base64
import hmac
import hashlib
import os

from base import BaseUploadBackend, _set_default_if_none, json

# AWS Options
ACCESS_KEY_ID       = getattr(settings, 'AWS_ACCESS_KEY_ID', None)
SECRET_ACCESS_KEY   = getattr(settings, 'AWS_SECRET_ACCESS_KEY', None)
BUCKET_NAME         = getattr(settings, 'AWS_BUCKET_NAME', None)
SECURE_URLS         = getattr(settings, 'AWS_S3_SECURE_URLS', False)
BUCKET_URL          = getattr(settings, 'AWS_BUCKET_URL', ('https://' if SECURE_URLS else 'http://') + BUCKET_NAME + '.s3.amazonaws.com')
DEFAULT_ACL         = getattr(settings, 'AWS_DEFAULT_ACL', 'public-read')
DEFAULT_KEY_PATTERN = getattr(settings, 'AWS_DEFAULT_KEY_PATTERN', '${targetname}')
DEFAULT_FORM_TIME   = getattr(settings, 'AWS_DEFAULT_FORM_LIFETIME', 36000) # 10 HOURS


class S3Backend(BaseUploadBackend):
    def __init__(self, request, options={}, post_data={}, conditions={}):
        self.conditions = conditions
        super(S3Backend, self).__init__(request, options, post_data)
    
    def get_target_url(self):
        return BUCKET_URL
    
    def build_options(self):
        self.options['forceIframeTransport'] = True
        self.options['fileObjName'] = 'file'
    
    def build_post_data(self):
        if 'folder' in self.options:
            key = os.path.join(self.options['folder'], DEFAULT_KEY_PATTERN)
        else:
            key = DEFAULT_KEY_PATTERN
        #_set_default_if_none(self.post_data, 'key', key) #this is set by update_post_params
        _set_default_if_none(self.post_data, 'acl', DEFAULT_ACL)
        
        try:
            _set_default_if_none(self.post_data, 'bucket', BUCKET_NAME)
        except ValueError:
            raise ImproperlyConfigured("Bucket name is a required property.")
 
        try:
            _set_default_if_none(self.post_data, 'AWSAccessKeyId', ACCESS_KEY_ID)
        except ValueError:
            raise ImproperlyConfigured("AWS Access Key ID is a required property.")

        self.conditions = self.build_conditions()

        if not SECRET_ACCESS_KEY:
            raise ImproperlyConfigured("AWS Secret Access Key is a required property.")
        
        expiration_time = datetime.utcnow() + timedelta(seconds=DEFAULT_FORM_TIME)
        self.policy_string = self.build_post_policy(expiration_time)
        self.policy = base64.b64encode(self.policy_string)
         
        self.signature = base64.encodestring(hmac.new(SECRET_ACCESS_KEY, self.policy, hashlib.sha1).digest()).strip()
        
        self.post_data['policy'] = self.policy
        self.post_data['signature'] = self.signature
    
    def build_conditions(self):
        conditions = list()
        
        #make s3 happy with uploadify
        conditions.append(['starts-with', '$targetname', '']) #variable introduced by this package
        conditions.append(['starts-with', '$targetpath', self.options['folder']])
        conditions.append({'success_action_status': '200'})
        
        #real conditions
        conditions.append(['starts-with', '$key', self.options['folder']])
        conditions.append({'bucket': self.post_data['bucket']})
        conditions.append({'acl': self.post_data['acl']})
        return conditions
    
    def build_post_policy(self, expiration_time):
        policy = {'expiration': expiration_time.strftime("%Y-%m-%dT%H:%M:%SZ"),
                  'conditions': self.conditions,}
        return json.dumps(policy)
    
    def update_post_params(self, params):
        #instruct s3 that our key is the targetpath
        self.build_post_data()
        params.update(self.post_data)
        params['key'] = params['targetpath']
        params['success_action_status'] = '200'

def _uri_encode(str):
    try:
        # The Uploadify flash component apparently decodes the scriptData once, so we need to encode twice here.
        return quote_plus(quote_plus(str, safe='~'), safe='~')
    except:
        raise ValueError




try:
    import importlib
except ImportError:
    from django.utils import importlib

def get_directupload_backend():
    from directupload.app_settings import DIRECT_UPLOAD_BACKEND
    module_name, class_name = DIRECT_UPLOAD_BACKEND.rsplit('.', 1)
    module = importlib.import_module(module_name)
    return getattr(module, class_name)




from django.conf import settings

DIRECT_UPLOAD_BACKEND = getattr(settings, 'DIRECT_UPLOAD_BACKEND', 'directupload.backends.djangoview.DjangoViewBackend')
#UPLOADIFY_BACKEND = getattr(settings, 'UPLOADIFY_BACKEND', 'directupload.backends.s3.S3UploadifyBackend')




from django.conf.urls.defaults import patterns, url

urlpatterns = patterns('directupload.views',
    url(r'^upload-options/$', 'upload_options_view', name='directupload-options'),
    url(r'^upload/$', 'upload_file', name='directupload-file'),
    url(r'^exists/$', 'determine_name', name='directupload-determine-name'),
)




from django.db import models
from django.contrib.admin import ModelAdmin

from widgets import DirectUploadClearableFileInput

class DirectUploadAdminMixin(object):
    def formfield_for_dbfield(self, db_field, **kwargs):
        if isinstance(db_field, models.FileField):
            return self.formfield_for_file_field(db_field, kwargs.pop('request', None), **kwargs)
        return ModelAdmin.formfield_for_dbfield(self, db_field, **kwargs)
    
    def formfield_for_file_field(self, db_field, request=None, **kwargs):
        """
        Get a form Field that is prepared for uploadify
        """
        kwargs['widget'] = DirectUploadClearableFileInput(db_field=db_field)
        return db_field.formfield(**kwargs)

def patch_admin():
    from django.contrib.admin.options import FORMFIELD_FOR_DBFIELD_DEFAULTS
    FORMFIELD_FOR_DBFIELD_DEFAULTS[models.ImageField] = {'widget': DirectUploadClearableFileInput}
    FORMFIELD_FOR_DBFIELD_DEFAULTS[models.FileField] = {'widget': DirectUploadClearableFileInput}




from django.conf import settings
from django.utils.crypto import constant_time_compare, salted_hmac

import base64

def b64_encode(s):
    return base64.urlsafe_b64encode(s).strip('=')

def b64_decode(s):
    pad = '=' * (-len(s) % 4)
    return base64.urlsafe_b64decode(s + pad)

def base64_hmac(salt, value, key):
    return b64_encode(salted_hmac(salt, value, key).digest())

class LegacySigner(object):
    '''
    Limited functionality signer
    '''
    def __init__(self, key=None, sep=':', salt=None):
        self.sep = sep
        self.key = key or settings.SECRET_KEY
        self.salt = salt or ('%s.%s' %
            (self.__class__.__module__, self.__class__.__name__))
    
    def signature(self, value):
        return base64_hmac(self.salt + 'signer', value, self.key)
    
    def sign(self, value):
        return '%s%s%s' % (value, self.sep, self.signature(value))

    def unsign(self, signed_value):
        if not self.sep in signed_value:
            raise ValueError('No "%s" found in value' % self.sep)
        value, sig = signed_value.rsplit(self.sep, 1)
        if constant_time_compare(sig, self.signature(value)):
            return value
        raise ValueError('Signature "%s" does not match' % sig)

try:
    from django.core.signing import Signer
except ImportError:
    Signer = LegacySigner




from django.conf import settings
from django.core.urlresolvers import reverse

from django.utils import simplejson as json

class BaseUploadBackend(object):
    def __init__(self, request, options={}, post_data={}):
        self.request = request
        self.options = getattr(settings, 'DEFAULT_DIRECTUPLOAD_OPTIONS', {})
        self.options.update(options)
        
        _set_default_if_none(self.options, 'url', self.get_target_url())
        _set_default_if_none(self.options, 'determineName', self.get_determine_name())
        
        self.build_options()
        self.post_data = post_data
    
    def get_determine_name(self):
        return reverse('directupload-determine-name')
    
    def get_target_url(self):
        pass
    
    def build_options(self):
        pass
    
    def build_post_data(self):
        pass
    
    def get_options_json(self):
        #self.options['postData'] = self.post_data
        subs = []
        out = json.dumps(self.options)
        
        for search, replace in subs:
            out = out.replace(search, replace)
            
        return out
    
    def update_post_params(self, params):
        self.build_post_data()
        params.update(self.post_data)

def _set_default_if_none(dict, key, default=None):
    if key not in dict:
        dict[key] = default




#!/usr/bin/env python

try:
    from setuptools import setup, find_packages
except ImportError:
    from distutils.core import setup, find_packages

version = '0.0.10'

setup(name='django-directupload',
      version=version,
      description='A Django application that offers direct uploading of files from your forms using Uploadify.',
      author='Jason Kraus',
      author_email='jasonk@cukerinteractive.com',
      url='https://github.com/cuker/django-directupload',
      packages=find_packages(exclude=['test_environment']),
      include_package_data = True,
      classifiers=['Development Status :: 4 - Beta',
                   'Environment :: Web Environment',
                   'Framework :: Django',
                   'Intended Audience :: Developers',
                   'License :: OSI Approved :: BSD License',
                   'Operating System :: OS Independent',
                   'Programming Language :: Python',
                   'Topic :: Utilities'],
      )




from django.forms.widgets import FileInput, ClearableFileInput
from django.core.files.storage import default_storage
from django.db.models.fields.files import FileField, FieldFile
from django.conf import settings

class DirectUploadInputMixin(object):
    db_field = None
    
    class Media: #this does not work for the admin as django ignores it [WTF]
        js = ('//ajax.googleapis.com/ajax/libs/jquery/1.7.1/jquery.min.js',
              settings.STATIC_URL+'directupload/js/vendor/jquery.ui.widget.js',
              settings.STATIC_URL+'directupload/js/jquery.fileupload.js',
              settings.STATIC_URL+'directupload/js/jquery.iframe-transport.js',
              settings.STATIC_URL+'directupload/widget.js',)
    
    def get_file_field(self):
        if self.db_field:
            return self.db_field
        return FileField(upload_to='', storage=default_storage)
    
    def value_from_datadict(self, data, files, name):
        "File widgets take data from FILES, not POST"
        if data.get(name, None):
            file_path = data[name]
            file_field = self.get_file_field()
            file_obj = file_field.storage.open(file_path)
            file_copy = FieldFile(None, file_field, file_path)#file_field.attr_class(None, file_field, file_path)
            file_copy.file = file_obj
            file_copy._committed = True
            return file_copy
        return None
    
    def prepare_attrs(self, attrs):
        attrs = attrs or {}
        file_field = self.get_file_field()
        attrs['class'] = 'directupload'
        attrs['data-upload-to'] = file_field.upload_to
        return attrs

class DirectUploadFileInput(DirectUploadInputMixin, FileInput):
    def __init__(self, *args, **kwargs):
        self.db_field = kwargs.pop('db_field', None)
        FileInput.__init__(self, *args, **kwargs)
    
    def value_from_datadict(self, data, files, name):
        file_obj = DirectUploadInputMixin.value_from_datadict(self, data, files, name)
        if file_obj:
            return file_obj
        return super(DirectUploadFileInput, self).value_from_datadict(data, files, name)
    
    def render(self, name, value, attrs=None):
        attrs = self.prepare_attrs(attrs)
        return super(DirectUploadFileInput, self).render(name, value, attrs)

class DirectUploadClearableFileInput(DirectUploadInputMixin, ClearableFileInput):
    def __init__(self, *args, **kwargs):
        self.db_field = kwargs.pop('db_field', None)
        ClearableFileInput.__init__(self, *args, **kwargs)
    
    def value_from_datadict(self, data, files, name):
        file_obj = DirectUploadInputMixin.value_from_datadict(self, data, files, name)
        if file_obj:
            return file_obj
        return super(DirectUploadClearableFileInput, self).value_from_datadict(data, files, name)
    
    def render(self, name, value, attrs=None):
        attrs = self.prepare_attrs(attrs)
        return super(DirectUploadClearableFileInput, self).render(name, value, attrs)

